import 'package:http/http.dart' as http;
import 'dart:convert';

class ApiClient {
  final String baseUrl = "https://web.mentor.iblai.tech/wp-json";

  // Login function
  Future<Map<String, dynamic>?> login(String username, String password) async {
    final response = await http.post(
      Uri.parse('$baseUrl/api/ibl/v1/authentication/login'),
      body: {'username': username, 'password': password, 'mobile': '1'},
    );

    var data = json.decode(response.body);

    // Extracting the token from the redirect_uri field in the response
    String? redirectUri = data['redirect_uri'];
    if (redirectUri != null) {
      String? token = Uri.parse(redirectUri).queryParameters['token'];
      if (token != null) {
        return {'token': token}; // Return the token as a JSON object
      }
    }
    return null; // Return null if redirectUri is not in the response or token is not found
  }

  // Register function
  Future<Map<String, dynamic>?> register(String username, String email, String name, String password) async {
    final response = await http.post(
      Uri.parse('$baseUrl/api/ibl/v1/authentication/register-user'),
      body: {
        'username': username,
        'email': email,
        'name': name,
        'password': password,
        'mobile': '1'
      },
    );

    var data = json.decode(response.body);

    // Extracting the token from the redirect_uri field in the response
    String? redirectUri = data['redirect_uri'];
    if (redirectUri != null) {
      String? token = Uri.parse(redirectUri).queryParameters['code'];
      if (token != null) {
        return {'token': token}; // Return the token as a JSON object
      }
    }
    return data; // Return null if redirectUri is not in the response or token is not found
  }
}


// void main() async {
//   ApiClient apiClient = ApiClient();

//   try {
//     Map<String, dynamic>? loginData = await apiClient.login("bimsara", "Bimsara@1234");
//     if (loginData != null) {
//         print(loginData);
//     //   String? loginToken = loginData['token'];
//     }
//   } catch (e) {
//     print("Error occurred: $e");
//   }
// }


void main() async {
  ApiClient apiClient = ApiClient();

  // Testing the register function
  try {
    Map<String, dynamic>? registerData = await apiClient.register("newuse8r1", "new1user@exampple.com", "New Ukser1", "User1Password123");
    if (registerData != null) {
      print("Register Data: $registerData");
      // Extract token or other relevant data from registerData
    }
  } catch (e) {
    print("Registration error occurred: $e");
  }

//   // Testing the login function
//   try {
//     Map<String, dynamic>? loginData = await apiClient.login("bimsara", "Bimsara@1234");
//     if (loginData != null) {
//         print("Login Data: $loginData");
//         // Extract token or other relevant data from loginData
//     }
//   } catch (e) {
//     print("Login error occurred: $e");
//   }
}
